//
// C++ Implementation: kpgtableindex
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtableindex.h"

// include files for Qt
#include <qtable.h>

// include files for KDE
#include <kdebug.h>
#include <klocale.h>

#include "kpgtableindexesfolder.h"
#include "../kpgsqldialog.h"


KPGTableIndex::KPGTableIndex(KPGTableIndexesFolder *parent, const QString name, pqxx::oid _oid)
  : KPGObject(parent, name, _oid)
{
	setPixmap(0, *m_pIconFolderRed);
}

KPGTableIndex::KPGTableIndex(KPGTableIndexesFolder *parent, KPGTableIndex *after, const QString name, pqxx::oid _oid)
  : KPGObject(parent, after, name, _oid)
{
	setPixmap(0, *m_pIconFolderRed);
}


KPGTableIndex::~KPGTableIndex()
{
}

// For PostgreSQL 7.x or older
void KPGTableIndex::setProperties(
		const QString strDescription,
		const QString strIndKey,
		bool bIsUnique,
		bool bIsPrimary,
		bool bIsClustered,
    const QString &strIndexdef
	)
{
	m_strDescription = strDescription;
	m_strIndKey = strIndKey;
	m_bIsUnique = bIsUnique;
	m_bIsPrimary = bIsPrimary;
	m_bIsClustered = bIsClustered;
  m_oidTablespace = 0;
  m_strIndexdef = strIndexdef;
}

// For PostgreSQL 8 or newer
void KPGTableIndex::setProperties(
    const QString strDescription,
    const QString strIndKey,
    bool bIsUnique,
    bool bIsPrimary,
    bool bIsClustered,
    pqxx::oid oidTablespace,    
    QString strTablespace,
    const QString &strIndexdef     
  )
{
  m_strDescription = strDescription;
  m_strIndKey = strIndKey;
  m_bIsUnique = bIsUnique;
  m_bIsPrimary = bIsPrimary;
  m_bIsClustered = bIsClustered;
  m_oidTablespace = oidTablespace;
  m_strTablespace = strTablespace;
  m_strIndexdef = strIndexdef;
}

void KPGTableIndex::displayStatistics(QTable *pTableStatistics)
{
  QString strQuery("SELECT indexrelname, idx_scan, idx_tup_read, idx_tup_fetch FROM pg_stat_all_indexes WHERE indexrelid = ");
  strQuery.append(QString("%1;").arg(m_oid));
  
  //--- Header field names
  QHeader* hHeader = pTableStatistics->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
  hHeader->setLabel(1, i18n("Value"));
  hHeader->setLabel(2, i18n("Description"));
  
  try
  {
    KPGConnection *pDbConnection = connection();
    pqxx::work pqxxXaction(*pDbConnection, "refresh table statistics");
    pqxx::result pqxxResultStatistics = pqxxXaction.Exec(strQuery.latin1());

    if(pqxxResultStatistics.size() != 1)
    {
      kdDebug() << "KPGTable::displayStatistics(): expect one row in result" << endl;
      return;
    }
    
    if(text(0) != pqxxResultStatistics[0]["indexrelname"].c_str())
    {
      kdDebug() << "KPGTable::displayStatistics(): table name not match" << endl;
      return;
    }
    
    int iRow = 0;
    
    //--- seq_scan
    pTableStatistics->setText(iRow, 0, i18n("idx_scan"));
    pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_scan"].c_str());
    pTableStatistics->setText(iRow++, 2, i18n("Total number of index scans"));
    
    //--- seq_scan
    pTableStatistics->setText(iRow, 0, i18n("idx_tup_read"));
    pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_tup_read"].c_str());
    pTableStatistics->setText(iRow++, 2, i18n("Total number of tuples read"));
    
    //--- idx_tup_fetch
    pTableStatistics->setText(iRow, 0, i18n("idx_tup_fetch"));
    pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_tup_fetch"].c_str());
    pTableStatistics->setText(iRow++, 2, i18n("Total number of sucessfully fetched heap tuples"));
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
    dlg.exec();
    return;
  } 
  
  for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableStatistics->adjustColumn(nCol);
  }
}

void KPGTableIndex::displayIoStatistics(QTable *pTableStatistics)
{
  QString strQuery("SELECT indexrelname, idx_blks_read, idx_blks_hit FROM pg_statio_all_indexes WHERE indexrelid = ");
  strQuery.append(QString("%1;").arg(m_oid));
  
  //--- Header field names
  QHeader* hHeader = pTableStatistics->horizontalHeader();

  hHeader->setLabel(0, i18n("Name"));
  hHeader->setLabel(1, i18n("Value"));
  hHeader->setLabel(2, i18n("Description"));

  try
  {
    KPGConnection *pDbConnection = connection();
    pqxx::work pqxxXaction(*pDbConnection, "refresh table statistics");
    pqxx::result pqxxResultStatistics = pqxxXaction.Exec(strQuery.latin1());

    if(pqxxResultStatistics.size() != 1)
    {
      kdDebug() << "KPGTable::displayStatistics(): expect one row in result" << endl;
      return;
    }
    
    if(text(0) != pqxxResultStatistics[0]["indexrelname"].c_str())
    {
      kdDebug() << "KPGTable::displayStatistics(): table name not match" << endl;
      return;
    }
    
    int iRow = 0;
    
    //--- seq_scan
    pTableStatistics->setText(iRow, 0, i18n("idx_blks_read"));
    pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_blks_read"].c_str());
    pTableStatistics->setText(iRow++, 2, i18n("Total number of disc blocks read"));
    
    //--- heap_blks_hit
    pTableStatistics->setText(iRow, 0, i18n("idx_blks_hit"));
    pTableStatistics->setText(iRow, 1, pqxxResultStatistics[0]["idx_blks_hit"].c_str());
    pTableStatistics->setText(iRow++, 2, i18n("Number of buffer hits"));
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
    dlg.exec();
    return;
  } 
  
  for(int nCol = 0; nCol < 3; nCol++)
  {
    pTableStatistics->adjustColumn(nCol);
  }
}



